@@ -32,7 +32,6 @@
     die unless @statprec  == @stattypes;
 
     $drh = undef;	# holds driver handle once initialised
-    $err = 0;		# The $DBI::err value
     #$gensym = "SYM000"; # used by st::execute() for filehandles
 
     sub driver{
@@ -59,13 +58,10 @@
 
     sub connect { # normally overridden, but a handy default
         my($drh, $dbname, $user, $auth)= @_;
-        my($this) = DBI::_new_dbh($drh, {
-	    'Name' => $dbname,
-	    'User' => $user,
-	    examplep_get_info => {},
-	    });
-	$this->STORE(Active => 1);
-        $this;
+        my ($outer, $dbh) = DBI::_new_dbh($drh, { Name => $dbname });
+        $dbh->STORE('Active', 1);
+        $dbh->{examplep_get_info} = {};
+        return $outer;
     }
 
     sub data_sources {
@@ -82,19 +78,25 @@
 
     sub prepare {
 	my($dbh, $statement)= @_;
+	my @fields;
+	my($fields, $dir) = $statement =~ m/^\s*select\s+(.*?)\s+from\s+(\S*)/i;
 
-	my($fields, $dir)
-		= $statement =~ m/^\s*select\s+(.*?)\s+from\s+(\S*)/i;
-	return $dbh->set_err(1, "Syntax error in select statement (\"$statement\")")
-		unless defined $fields and defined $dir;
-
-	my ($outer, $inner) = DBI::_new_sth($dbh, {
-	    'Statement'     => $statement,
-	}, ['example implementors private data '.__PACKAGE__]);
-
-	my @fields = ($fields eq '*')
+	if (defined $fields and defined $dir) {
+	    @fields = ($fields eq '*')
 			? keys %DBD::ExampleP::statnames
 			: split(/\s*,\s*/, $fields);
+	}
+	else {
+	    return $dbh->set_err(1, "Syntax error in select statement (\"$statement\")")
+		unless $statement =~ m/^\s*set\s+/;
+	    # the SET syntax is just a hack so the ExampleP driver can
+	    # be used to test non-select statements.
+	    # No we have DBI::DBM etc ExampleP should be deprecated
+	}
+
+	my ($outer, $sth) = DBI::_new_sth($dbh, {
+	    'Statement'     => $statement,
+	}, ['example implementors private data '.__PACKAGE__]);
 
 	my @bad = map {
 	    defined $DBD::ExampleP::statnames{$_} ? () : $_
@@ -102,14 +104,16 @@
 	return $dbh->set_err(1, "Unknown field names: @bad")
 		if @bad;
 
-	$inner->{dbd_param} = [];
-	@{ $inner->{'dbd_param'} } = ($dir) if $dir !~ /\?/;
-
-	$outer->STORE('NAME' => \@fields);
-	$outer->STORE('NULLABLE' => [ (0) x @fields ]);
 	$outer->STORE('NUM_OF_FIELDS' => scalar(@fields));
-	$outer->STORE('NUM_OF_PARAMS' => ($dir !~ /\?/) ? 0 : 1);
-	$outer->STORE('SCALE'     => [ (0) x @fields ] );
+
+	$sth->{dbd_ex_dir} = $dir if defined($dir) && $dir !~ /\?/;
+	$outer->STORE('NUM_OF_PARAMS' => ($dir) ? $dir =~ tr/?/?/ : 0);
+
+	if (@fields) {
+	    $outer->STORE('NAME'     => \@fields);
+	    $outer->STORE('NULLABLE' => [ (0) x @fields ]);
+	    $outer->STORE('SCALE'    => [ (0) x @fields ]);
+	}
 
 	$outer;
     }
@@ -285,10 +289,12 @@
 	}
 
 	my $dbd_param = $sth->{'dbd_param'} || [];
-	return $sth->set_err(2, @$dbd_param." values bound when 1 expected")
-	    unless @$dbd_param == 1;
+	return $sth->set_err(2, @$dbd_param." values bound when $sth->{NUM_OF_PARAMS} expected")
+	    unless @$dbd_param == $sth->{NUM_OF_PARAMS};
 
-	$dir = $dbd_param->[0];
+	return 0 unless $sth->{NUM_OF_FIELDS}; # not a select
+
+	$dir = $dbd_param->[0] || $sth->{dbd_ex_dir};
 	return $sth->set_err(2, "No bind parameter supplied")
 	    unless defined $dir;
 
@@ -393,7 +399,10 @@
 	return $sth->SUPER::STORE($attrib, $value);
     }
 
-    sub DESTROY { undef }
+    sub DESTROY {
+	my $sth = shift;
+	$sth->finish if $sth->SUPER::FETCH('Active');
+    }
 
     *parse_trace_flag = \&DBD::ExampleP::db::parse_trace_flag;
 }
