@@ -14,7 +14,6 @@
 #   License or the Artistic License, as specified in the Perl README file.
 
     $drh = undef;	# holds driver handle once initialised
-    $err = 0;		# The $DBI::err value
 
     sub driver{
 	return $drh if $drh;
@@ -107,10 +106,6 @@
 	return undef;
     }
 
-    sub finish {
-	my($sth) = @_;
-    }
-
     sub FETCH {
 	my ($sth, $attrib) = @_;
 	# would normally validate and only fetch known attributes
