@@ -31,7 +31,7 @@ use RPC::PlClient 0.2000; # XXX change to 0.2017 once it's released
 
 package DBD::Proxy;
 
-use vars qw($VERSION $err $errstr $drh %ATTR);
+use vars qw($VERSION $drh %ATTR);
 
 $VERSION = "0.2004";
 
@@ -222,6 +222,7 @@ use vars qw(%ATTR $AUTOLOAD);
     Statement => 'local',
     Driver => 'local',
     dbi_connect_closure => 'local',
+    Username => 'local',
 );
 
 sub AUTOLOAD {
@@ -236,12 +237,13 @@ sub AUTOLOAD {
 	  'type' => $type,
 	  'h' => "DBI::_::$type"
 	);
-    local $SIG{__DIE__} = 'DEFAULT';
     my $method_code = UNIVERSAL::can($expand{'h'}, $method) ?
 	q/package ~class~;
           sub ~method~ {
             my $h = shift;
-	    my @result = eval { $h->{'proxy_~type~h'}->~method~(@_) };
+	    my @result = wantarray
+		? eval {        $h->{'proxy_~type~h'}->~method~(@_) }
+		: eval { scalar $h->{'proxy_~type~h'}->~method~(@_) };
             return DBD::Proxy::proxy_set_err($h, $@) if $@;
             wantarray ? @result : $result[0];
           }
@@ -249,19 +251,23 @@ sub AUTOLOAD {
         q/package ~class~;
 	  sub ~method~ {
 	    my $h = shift;
-	    my @result = eval { $h->{'proxy_~type~h'}->func(@_, '~method~') };
+	    my @result = wantarray
+		? eval {        $h->{'proxy_~type~h'}->func(@_, '~method~') }
+		: eval { scalar $h->{'proxy_~type~h'}->func(@_, '~method~') };
 	    return DBD::Proxy::proxy_set_err($h, $@) if $@;
 	    wantarray ? @result : $result[0];
           }
          /;
     $method_code =~ s/\~(\w+)\~/$expand{$1}/eg;
-    eval $method_code;
-    die $@ if $@;
+    local $SIG{__DIE__} = 'DEFAULT';
+    my $err = do { local $@; eval $method_code.2; $@ };
+    die $err if $err;
     goto &$AUTOLOAD;
 }
 
 sub DESTROY {
     my $dbh = shift;
+    local $@ if $@;	# protect $@
     $dbh->disconnect if $dbh->SUPER::FETCH('Active');
 }
 
@@ -301,6 +307,7 @@ sub STORE ($$$) {
 
     if ($type eq 'remote'  ||  $type eq 'cached') {
         local $SIG{__DIE__} = 'DEFAULT';
+	local $@;
 	my $result = eval { $dbh->{'proxy_dbh'}->STORE($attr => $val) };
 	return DBD::Proxy::proxy_set_err($dbh, $@) if $@; # returns undef
 	$dbh->{$attr} = $val if $type eq 'cached';
@@ -321,6 +328,7 @@ sub FETCH ($$) {
     return $dbh->SUPER::FETCH($attr) unless $type eq 'remote';
 
     local $SIG{__DIE__} = 'DEFAULT';
+    local $@;
     my $result = eval { $dbh->{'proxy_dbh'}->FETCH($attr) };
     return DBD::Proxy::proxy_set_err($dbh, $@) if $@;
     return $result;
@@ -340,6 +348,7 @@ sub prepare ($$;$) {
       $sth->{'proxy_attr_cache'} = {cache_filled => 0};
       my $rdbh = $dbh->{'proxy_dbh'};
       local $SIG{__DIE__} = 'DEFAULT';
+      local $@;
       my $rsth = eval { $rdbh->prepare($sth->{'Statement'}, $sth->{'proxy_attr'}, undef, $proto_ver) };
       return DBD::Proxy::proxy_set_err($sth, $@) if $@;
       return DBD::Proxy::proxy_set_err($sth, "Constructor didn't return a handle: $rsth")
@@ -378,6 +387,7 @@ sub quote {
     # for example.
     # Jochen
     local $SIG{__DIE__} = 'DEFAULT';
+    local $@;
     my $result = eval { $dbh->{'proxy_dbh'}->quote(@_) };
     return DBD::Proxy::proxy_set_err($dbh, $@) if $@;
     return $result;
@@ -388,6 +398,7 @@ sub table_info {
     my $rdbh = $dbh->{'proxy_dbh'};
     #warn "table_info(@_)";
     local $SIG{__DIE__} = 'DEFAULT';
+    local $@;
     my($numFields, $names, $types, @rows) = eval { $rdbh->table_info(@_) };
     return DBD::Proxy::proxy_set_err($dbh, $@) if $@;
     my ($sth, $inner) = DBI::_new_sth($dbh, {
@@ -421,6 +432,7 @@ sub tables {
 sub type_info_all {
     my $dbh = shift;
     local $SIG{__DIE__} = 'DEFAULT';
+    local $@;
     my $result = eval { $dbh->{'proxy_dbh'}->type_info_all(@_) };
     return DBD::Proxy::proxy_set_err($dbh, $@) if $@;
     return $result;
@@ -472,6 +484,7 @@ sub execute ($@) {
     my ($numRows, @outData);
 
     local $SIG{__DIE__} = 'DEFAULT';
+    local $@;
     if ( $proto_ver > 1 ) {
       ($numRows, @outData) = eval { $rsth->execute($params, $proto_ver) };
       return DBD::Proxy::proxy_set_err($sth, $@) if $@;
@@ -552,6 +565,7 @@ sub fetch ($) {
 	}
 	my $num_rows = $sth->FETCH('RowCacheSize') || 20;
 	local $SIG{__DIE__} = 'DEFAULT';
+	local $@;
 	my @rows = eval { $rsth->fetch($num_rows) };
 	return DBD::Proxy::proxy_set_err($sth, $@) if $@;
 	unless (@rows == $num_rows) {
@@ -585,6 +599,7 @@ sub finish ($) {
 	: $sth->FETCH('Database')->{'proxy_no_finish'};
     unless ($no_finish) {
         local $SIG{__DIE__} = 'DEFAULT';
+	local $@;
 	my $result = eval { $rsth->finish() };
 	return DBD::Proxy::proxy_set_err($sth, $@) if $@;
 	return $result;
@@ -608,6 +623,7 @@ sub STORE ($$$) {
     if ($type eq 'remote') {
 	my $rsth = $sth->{'proxy_sth'}  or  return undef;
         local $SIG{__DIE__} = 'DEFAULT';
+	local $@;
 	my $result = eval { $rsth->STORE($attr => $val) };
 	return DBD::Proxy::proxy_set_err($sth, $@) if ($@);
 	return $result;
@@ -638,6 +654,7 @@ sub FETCH ($$) {
     if ($type ne 'local') {
 	my $rsth = $sth->{'proxy_sth'}  or  return undef;
         local $SIG{__DIE__} = 'DEFAULT';
+	local $@;
 	my $result = eval { $rsth->FETCH($attr) };
 	return DBD::Proxy::proxy_set_err($sth, $@) if $@;
 	return $result;
@@ -658,7 +675,8 @@ sub bind_param ($$$@) {
 *bind_param_inout = \&bind_param;
 
 sub DESTROY {
-    # Just to avoid autoloading DESTROY ...
+    my $sth = shift;
+    $sth->finish if $sth->SUPER::FETCH('Active');
 }
 
 
@@ -708,7 +726,7 @@ I<Note that the connect_cached method is new and still experimental.>
 Before connecting to a remote database, you must ensure, that a Proxy
 server is running on the remote machine. There's no default port, so
 you have to ask your system administrator for the port number. See
-L<DBI::ProxyServer(3)> for details.
+L<DBI::ProxyServer> for details.
 
 Say, your Proxy server is running on machine "alpha", port 3334, and
 you'd like to connect to an ODBC database called "mydb" as user "joe"
@@ -725,7 +743,7 @@ You see, this is mainly the same. The DBD::Proxy module will create a
 connection to the Proxy server on "alpha" which in turn will connect
 to the ODBC database.
 
-Refer to the L<DBI(3)> documentation on the C<connect> method for a way
+Refer to the L<DBI> documentation on the C<connect> method for a way
 to automatically use DBD::Proxy without having to change your code.
 
 DBD::Proxy's DSN string has the format
@@ -775,7 +793,7 @@ by executing
     $cipherRef = $class->new(pack("H*", $key));
 
 and pass this object to the RPC::PlClient module when creating a
-client. See L<RPC::PlClient(3)>. Example:
+client. See L<RPC::PlClient>. Example:
 
     cipher=IDEA;key=97cd2375efa329aceef2098babdc9721
 
@@ -787,7 +805,7 @@ less secure than the usercipher/userkey secret and readable by anyone.
 The usercipher/userkey secret is B<your> private secret.
 
 Of course encryption requires an appropriately configured server. See
-<DBD::ProxyServer(3)/CONFIGURATION FILE>.
+<DBD::ProxyServer/CONFIGURATION FILE>.
 
 =item debug
 
@@ -834,7 +852,7 @@ and fast CGI applications.
 This attribute can be used to reduce network traffic: By default calls
 to $dbh->quote() are passed to the remote driver.  Of course this slows
 down things quite a lot, but is the safest default behaviour.
-  
+
 However, if you set the I<proxy_quote> attribute to the value 'C<local>'
 either in the database handle or in the statement handle, and the call
 to quote has only one parameter, then the local default DBI quote
@@ -864,13 +882,13 @@ executing the above example in two steps:
 
 =over
 
-=item 1.)
- 
+=item 1
+
 The first step is fetching the value of the key "csv_tables" in the
 handle $dbh. The value returned is complex, a hash ref.
- 
-=item 2.)
- 
+
+=item 2
+
 The second step is storing some value (the right hand side of the
 assignment) as the key "passwd" in the hash ref from step 1.
 
@@ -914,6 +932,6 @@ is granted to Tim Bunce for distributing this as a part of the DBI.
 
 =head1 SEE ALSO
 
-L<DBI(3)>, L<RPC::PlClient(3)>, L<Storable(3)>
+L<DBI>, L<RPC::PlClient>, L<Storable>
 
 =cut
