@@ -1,5 +1,29 @@
+# $Id: ODBC.pm,v 1.3 2003/02/26 17:56:01 timbo Exp $
+#
+# Copyright (c) 2002  Tim Bunce  Ireland
+#
+# Constant data describing Microsoft ODBC info types and return values
+# for the SQLGetInfo() method of ODBC.
+#
+# You may distribute under the terms of either the GNU General Public
+# License or the Artistic License, as specified in the Perl README file.
+
 package DBI::Const::GetInfo::ODBC;
 
+=head1 NAME
+
+DBI::Const::GetInfo::ODBC - ODBC Constants for GetInfo
+
+=head1 SYNOPSIS
+
+ The API for this module is private and subject to change.
+
+=head1 DESCRIPTION
+
+Information requested by GetInfo().
+
+The API for this module is private and subject to change.   
+
 =head1 REFERENCES
 
   MDAC SDK 2.6
@@ -8,14 +32,11 @@ package DBI::Const::GetInfo::ODBC;
   sql.h
   sqlext.h
 
-=head1 Constants
-
-=head2 %InfoTypes
-
-Information requested by GetInfo().
-
 =cut
 
+my
+$VERSION = sprintf "%d.%02d", '$Revision: 1.3 $ ' =~ /(\d+)\.(\d+)/;
+
 %InfoTypes =
 (
   SQL_ACCESSIBLE_PROCEDURES           =>    20
@@ -78,6 +99,7 @@ Information requested by GetInfo().
 , SQL_CURSOR_SENSITIVITY              => 10001
 , SQL_DATA_SOURCE_NAME                =>     2
 , SQL_DATA_SOURCE_READ_ONLY           =>    25
+, SQL_DATABASE_NAME                   =>    16 
 , SQL_DATETIME_LITERALS               =>   119
 , SQL_DBMS_NAME                       =>    17
 , SQL_DBMS_VER                        =>    18
@@ -306,6 +328,7 @@ See: mk:@MSITStore:X:\dm\cli\mdac\sdk26\Docs\odbc.chm::/htm/odbcsqlgetinfo.htm
 , SQL_CURSOR_SENSITIVITY              => 'SQLUINTEGER'         # 10001
 , SQL_DATA_SOURCE_NAME                => 'SQLCHAR'             #     2
 , SQL_DATA_SOURCE_READ_ONLY           => 'SQLCHAR'             #    25
+, SQL_DATABASE_NAME                   => 'SQLCHAR'             #    16 
 , SQL_DATETIME_LITERALS               => 'SQLUINTEGER bitmask' #   119
 , SQL_DBMS_NAME                       => 'SQLCHAR'             #    17
 , SQL_DBMS_VER                        => 'SQLCHAR'             #    18
