@@ -30,12 +30,12 @@ You can also activate DBI::ProfileDumper from within your code:
   # another way to say it
   use DBI::Profile qw(DBIprofile_Statement);
   $dbh->{Profile} = DBI::ProfileDumper->new(
-                      { Path => [ DBIprofile_Statement ]
-                        File => 'dbi.prof' });
+                        Path => [ DBIprofile_Statement ]
+                        File => 'dbi.prof' );
 
   # using a custom path
-  $dbh->{Profile} = DBI::ProfileDumper->new({ Path => [ "foo", "bar" ],
-                                              File => 'dbi.prof' });
+  $dbh->{Profile} = DBI::ProfileDumper->new( Path => [ "foo", "bar" ],
+                                             File => 'dbi.prof' );
 
 
 =head1 DESCRIPTION
@@ -61,11 +61,11 @@ can construct the DBI::ProfileDumper object yourself:
 
   use DBI::Profile qw(DBIprofile_Statement);
   $dbh->{Profile} = DBI::ProfileDumper->new(
-                      { Path => [ DBIprofile_Statement ]
-                        File => 'dbi.prof' });
+                        Path => [ DBIprofile_Statement ]
+                        File => 'dbi.prof' );
 
 The C<Path> option takes the same values as in
-L<DBI::Profile|DBI:Profile>.  The C<File> option gives the name of the
+L<DBI::Profile>.  The C<File> option gives the name of the
 file where results will be collected.  If it already exists it will be
 overwritten.
 
@@ -107,7 +107,7 @@ newlines, the profile data forms the body of the file.  For example:
   DBI::ProfileDumper 1.0
   Path = [ DBIprofile_Statement, DBIprofile_MethodName ]
   Program = t/42profile_data.t
-  
+
   + 1 SELECT name FROM users WHERE id = ?
   + 2 prepare
   = 1 0.0312958955764771 0.000490069389343262 0.000176072120666504 0.00140702724456787 1023115819.83019 1023115819.86576
@@ -158,6 +158,7 @@ use vars qw(@ISA $VERSION);
 $VERSION = "1.0";
 
 use Carp qw(croak);
+use Symbol;
 
 # validate params and setup default
 sub new {
@@ -175,7 +176,7 @@ sub flush_to_disk {
     my $self = shift;
     my $data = $self->{Data};
 
-    my $fh;
+    my $fh = gensym;
     if ($self->{_wrote_header}) {
         # append more data to the file
         open($fh, ">>$self->{File}") 
