@@ -1,4 +1,4 @@
-#	$Header: /home/timbo/dbi/lib/DBI/RCS/ProxyServer.pm,v 11.6 2002/10/29 10:00:44 timbo Exp $
+#	$Header: /home/timbo/dbi/lib/DBI/RCS/ProxyServer.pm,v 11.9 2003/05/14 11:08:17 timbo Exp $
 # -*- perl -*-
 #
 #   DBI::ProxyServer - a proxy server for DBI drivers
@@ -26,19 +26,13 @@ require 5.004;
 use strict;
 
 use RPC::PlServer 0.2001;
-require DBI;
+# require DBI; # deferred till AcceptVersion() to aid threading
 require Config;
 
 
 package DBI::ProxyServer;
 
 
-my $haveFileSpec = eval { require File::Spec };
-my $tmpDir = $haveFileSpec ? File::Spec->tmpdir() :
-    ($ENV{'TMP'} || $ENV{'TEMP'} || '/tmp');
-my $defaultPidFile = $haveFileSpec ?
-    File::Spec->catdir($tmpDir, "dbiproxy.pid") : "/tmp/dbiproxy.pid";
-
 
 ############################################################################
 #
@@ -90,7 +84,8 @@ my %DEFAULT_SERVER_OPTIONS;
 	    'func' => 1,
 	    'quote' => 1,
 	    'type_info_all' => 1,
-            'table_info' => 1
+	    'table_info' => 1,
+	    'disconnect' => 1,
 	    },
 	'DBI::ProxyServer::st' => {
 	    'execute' => 1,
@@ -108,7 +103,8 @@ my %DEFAULT_SERVER_OPTIONS;
     } else {
 	$o->{'mode'} = 'single';
     }
-    $o->{'pidfile'}    = $defaultPidFile;
+    # No pidfile by default, configuration must provide one if needed
+    $o->{'pidfile'}    = 'none';
     $o->{'user'}       = undef;
 };
 
@@ -165,6 +161,8 @@ sub AcceptApplication {
 
 sub AcceptVersion {
     my $self = shift; my $version = shift;
+    require DBI;
+    DBI::ProxyServer->init_rootclass();
     $DBI::VERSION >= $version;
 }
 
@@ -220,7 +218,7 @@ sub CallMethod {
     my $msg = $@;
     undef $dbh->{'private_server'};
     if ($msg) {
-	$server->Error($msg);
+	$server->Debug("CallMethod died with: $@");
 	die $msg;
     } else {
 	$server->Debug("CallMethod: <= " . join(",", @result));
@@ -243,8 +241,6 @@ sub main {
 #
 ############################################################################
 
-DBI::ProxyServer->init_rootclass();
-
 package DBI::ProxyServer::dr;
 
 @DBI::ProxyServer::dr::ISA = qw(DBI::dr);
@@ -374,13 +370,11 @@ __END__
 
 DBI::ProxyServer - a server for the DBD::Proxy driver
 
-
 =head1 SYNOPSIS
 
     use DBI::ProxyServer;
     DBI::ProxyServer::main(@ARGV);
 
-
 =head1 DESCRIPTION
 
 DBI::Proxy Server is a module for implementing a proxy for the DBI proxy
@@ -401,18 +395,17 @@ query restrictions: You can define a set of queries that a client may
 execute and restrict access to those. (Requires a DBI driver that supports
 parameter binding.) See L</CONFIGURATION FILE>.
 
-The provided driver script, L<dbiproxy(1)>, may either be used as it is or
+The provided driver script, L<dbiproxy>, may either be used as it is or
 used as the basis for a local version modified to meet your needs.
 
 =head1 OPTIONS
 
 When calling the DBI::ProxyServer::main() function, you supply an
-array of options. (@ARGV, the array of command line options is used,
-if you don't.) These options are parsed by the Getopt::Long module.
+array of options. These options are parsed by the Getopt::Long module.
 The ProxyServer inherits all of RPC::PlServer's and hence Net::Daemon's
 options and option handling, in particular the ability to read
 options from either the command line or a config file. See
-L<RPC::PlServer(3)>. See L<Net::Daemon(3)>. Available options include
+L<RPC::PlServer>. See L<Net::Daemon>. Available options include
 
 =over 4
 
@@ -448,8 +441,7 @@ certain directory tree only after logging in. See also the --group and
 
 An array ref with a list of clients. Clients are hash refs, the attributes
 I<accept> (0 for denying access and 1 for permitting) and I<mask>, a Perl
-regular expression for the clients IP number or its host name. See
-L<"Access control"> below.
+regular expression for the clients IP number or its host name.
 
 =item I<configfile> (B<--configfile=file>)
 
@@ -465,7 +457,7 @@ level "debug" are created.
 
 =item I<facility> (B<--facility=mode>)
 
-(UNIX only) Facility to use for L<Sys::Syslog (3)>. The default is
+(UNIX only) Facility to use for L<Sys::Syslog>. The default is
 B<daemon>.
 
 =item I<group> (B<--group=gid>)
@@ -493,7 +485,7 @@ must be given somehow, as there's no default.
 Be default logging messages will be written to the syslog (Unix) or
 to the event log (Windows NT). On other operating systems you need to
 specify a log file. The special value "STDERR" forces logging to
-stderr. See L<Net::Daemon::Log(3)> for details.
+stderr. See L<Net::Daemon::Log> for details.
 
 =item I<mode> (B<--mode=modename>)
 
@@ -519,7 +511,7 @@ mode with "--mode=single".
 =item I<pidfile> (B<--pidfile=file>)
 
 (UNIX only) If this option is present, a PID file will be created at the
-given location.
+given location. Default is to not create a pidfile.
 
 =item I<user> (B<--user=uid>)
 
@@ -595,6 +587,275 @@ or
 which in fact are "SELECT * FROM foo" or "INSERT INTO foo VALUES (?, ?, ?)".
 
 
+=head1 Proxyserver Configuration file (bigger example)
+
+This section tells you how to restrict a DBI-Proxy: Not every user from
+every workstation shall be able to execute every query.
+
+There is a perl program "dbiproxy" which runs on a machine which is able
+to connect to all the databases we wish to reach. All Perl-DBD-drivers must
+be installed on this machine. You can also reach databases for which drivers 
+are not available on the machine where you run the programm querying the 
+database, e.g. ask MS-Access-database from Linux.
+
+Create a configuration file "proxy_oracle.cfg" at the dbproxy-server:
+
+    {
+	# This shall run in a shell or a DOS-window 
+	# facility => 'daemon',
+	pidfile => 'your_dbiproxy.pid',
+	logfile => 1,
+	debug => 0,
+	mode => 'single',
+	localport => '12400',
+
+	# Access control, the first match in this list wins!
+	# So the order is important
+	clients => [
+		# hint to organize:
+		# the most specialized rules for single machines/users are 1st
+		# then the denying rules
+		# the the rules about whole networks
+
+		# rule: internal_webserver
+		# desc: to get statistical information
+		{
+			# this IP-address only is meant
+			mask => '^10\.95\.81\.243$',
+			# accept (not defer) connections like this
+			accept => 1,
+			# only users from this list 
+			# are allowed to log on
+			users => [ 'informationdesk' ],
+			# only this statistical query is allowed
+			# to get results for a web-query
+			sql => {
+				alive => 'select count(*) from dual',
+				statistic_area => 'select count(*) from e01admin.e01e203 where geb_bezei like ?',
+			}
+		},
+
+		# rule: internal_bad_guy_1
+		{
+			mask => '^10\.95\.81\.1$',
+			accept => 0,
+		},
+
+		# rule: employee_workplace
+		# desc: get detailled informations
+		{
+			# any IP-address is meant here
+			mask => '^10\.95\.81\.(\d+)$',
+			# accept (not defer) connections like this
+			accept => 1,
+			# only users from this list 
+			# are allowed to log on
+			users => [ 'informationdesk', 'lippmann' ],
+			# all these queries are allowed:
+			sql => {
+				search_city => 'select ort_nr, plz, ort from e01admin.e01e200 where plz like ?',
+				search_area => 'select gebiettyp, geb_bezei from e01admin.e01e203 where geb_bezei like ? or geb_bezei like ?',
+			}
+		},
+
+		# rule: internal_bad_guy_2 
+		# This does NOT work, because rule "employee_workplace" hits
+		# with its ip-address-mask of the whole network
+		{
+			# don't accept connection from this ip-address
+			mask => '^10\.95\.81\.5$',
+			accept => 0,
+		}
+	]
+    }
+
+Start the proxyserver like this:
+
+	rem well-set Oracle_home needed for Oracle
+	set ORACLE_HOME=d:\oracle\ora81
+	dbiproxy --configfile proxy_oracle.cfg
+
+
+=head2 Testing the connection from a remote machine
+
+Call a programm "dbish" from your commandline. I take the machine from rule "internal_webserver"
+
+	dbish "dbi:Proxy:hostname=oracle.zdf;port=12400;dsn=dbi:Oracle:e01" informationdesk xxx
+
+There will be a shell-prompt:
+
+	informationdesk@dbi...> alive
+
+	Current statement buffer (enter '/'...):
+	alive
+
+	informationdesk@dbi...> /
+	COUNT(*)
+	'1'
+	[1 rows of 1 fields returned]
+
+
+=head2 Testing the connection with a perl-script
+
+Create a perl-script like this:
+
+	# file: oratest.pl
+	# call me like this: perl oratest.pl user password
+
+	use strict;
+	use DBI;
+
+	my $user = shift || die "Usage: $0 user password";
+	my $pass = shift || die "Usage: $0 user password";
+	my $config = {
+		dsn_at_proxy => "dbi:Oracle:e01",
+		proxy => "hostname=oechsle.zdf;port=12400",
+	};
+	my $dsn = sprintf "dbi:Proxy:%s;dsn=%s",
+		$config->{proxy},
+		$config->{dsn_at_proxy};
+
+	my $dbh = DBI->connect( $dsn, $user, $pass )
+		|| die "connect did not work: $DBI::errstr";
+
+	my $sql = "search_city";
+	printf "%s\n%s\n%s\n", "="x40, $sql, "="x40;
+	my $cur = $dbh->prepare($sql);
+	$cur->bind_param(1,'905%');
+	&show_result ($cur);
+
+	my $sql = "search_area";
+	printf "%s\n%s\n%s\n", "="x40, $sql, "="x40;
+	my $cur = $dbh->prepare($sql);
+	$cur->bind_param(1,'Pfarr%');
+	$cur->bind_param(2,'Bronnamberg%');
+	&show_result ($cur);
+
+	my $sql = "statistic_area";
+	printf "%s\n%s\n%s\n", "="x40, $sql, "="x40;
+	my $cur = $dbh->prepare($sql);
+	$cur->bind_param(1,'Pfarr%');
+	&show_result ($cur);
+
+	$dbh->disconnect;
+	exit;
+
+
+	sub show_result {
+		my $cur = shift;
+		unless ($cur->execute()) {
+			print "Could not execute\n"; 
+			return; 
+		}
+
+		my $rownum = 0;
+		while (my @row = $cur->fetchrow_array()) {
+			printf "Row is: %s\n", join(", ",@row);
+			if ($rownum++ > 5) {
+				print "... and so on\n";
+				last;
+			}	
+		}
+		$cur->finish;
+	}
+
+The result
+
+	C:\>perl oratest.pl informationdesk xxx
+	========================================
+	search_city
+	========================================
+	Row is: 3322, 9050, Chemnitz
+	Row is: 3678, 9051, Chemnitz
+	Row is: 10447, 9051, Chemnitz
+	Row is: 12128, 9051, Chemnitz
+	Row is: 10954, 90513, Zirndorf
+	Row is: 5808, 90513, Zirndorf
+	Row is: 5715, 90513, Zirndorf
+	... and so on
+	========================================
+	search_area
+	========================================
+	Row is: 101, Bronnamberg
+	Row is: 400, Pfarramt Zirndorf
+	Row is: 400, Pfarramt Rosstal
+	Row is: 400, Pfarramt Oberasbach
+	Row is: 401, Pfarramt Zirndorf
+	Row is: 401, Pfarramt Rosstal
+	========================================
+	statistic_area
+	========================================
+	DBD::Proxy::st execute failed: Server returned error: Failed to execute method CallMethod: Unknown SQL query: statistic_area at E:/Perl/site/lib/DBI/ProxyServer.pm line 258.
+	Could not execute
+
+
+=head2 How the configuration works
+
+The most important section to control access to your dbi-proxy is "client=>"
+in the file "proxy_oracle.cfg":
+
+Controlling which person at which machine is allowed to access
+
+=over 4
+
+=item * "mask" is a perl regular expression against the plain ip-address of the machine which wishes to connect _or_ the reverse-lookup from a nameserver.
+
+=item * "accept" tells the dbiproxy-server wether ip-adresse like in "mask" are allowed to connect or not (0/1)
+
+=item * "users" is a reference to a list of usernames which must be matched, this is NOT a regular expression.
+
+=back
+
+Controlling which SQL-statements are allowed
+
+You can put every SQL-statement you like in simply ommiting "sql => ...", but the more important thing is to restrict the connection so that only allowed queries are possible.
+
+If you include an sql-section in your config-file like this:
+
+	sql => {
+		alive => 'select count(*) from dual',
+		statistic_area => 'select count(*) from e01admin.e01e203 where geb_bezei like ?',
+	}
+
+The user is allowed to put two queries against the dbi-proxy. The queries are _not_ "select count(*)...", the queries are "alive" and "statistic_area"! These keywords are replaced by the real query. So you can run a query for "alive":
+
+	my $sql = "alive";
+	my $cur = $dbh->prepare($sql);
+	...
+
+The flexibility is that you can put parameters in the where-part of the query so the query are not static. Simply replace a value in the where-part of the query through a question mark and bind it as a parameter to the query. 
+
+	my $sql = "statistic_area";
+	my $cur = $dbh->prepare($sql);
+	$cur->bind_param(1,'905%');
+	# A second parameter would be called like this:
+	# $cur->bind_param(2,'98%');
+
+The result is this query:
+
+	select count(*) from e01admin.e01e203 
+	where geb_bezei like '905%'
+
+Don't try to put parameters into the sql-query like this:
+
+	# Does not work like you think.
+	# Only the first word of the query is parsed,
+	# so it's changed to "statistic_area", the rest is omitted.
+	# You _have_ to work with $cur->bind_param.
+	my $sql = "statistic_area 905%";
+	my $cur = $dbh->prepare($sql);
+	...
+
+
+=head2 Problems
+
+=over 4
+
+=item * I don't know how to restrict users to special databases.
+
+=item * I don't know how to pass query-parameters via dbish
+
+=back
 
 
 =head1 AUTHOR
@@ -615,6 +876,6 @@ the DBI.
 
 =head1 SEE ALSO
 
-L<dbiproxy(1)>, L<DBD::Proxy(3)>, L<DBI(3)>, L<RPC::PlServer(3)>,
-L<RPC::PlClient(3)>, L<Net::Daemon(3)>, L<Net::Daemon::Log(3)>,
-L<Sys::Syslog(3)>, L<Win32::EventLog(3)>, L<syslog(2)>
+L<dbiproxy>, L<DBD::Proxy>, L<DBI>, L<RPC::PlServer>,
+L<RPC::PlClient>, L<Net::Daemon>, L<Net::Daemon::Log>,
+L<Sys::Syslog>, L<Win32::EventLog>, L<syslog>
