@@ -23,12 +23,20 @@ use strict;
 use Carp;
 require Symbol;
 
+require utf8;
+*utf8::is_utf8 = sub { # hack for perl 5.6
+    require bytes;
+    return unless defined $_[0];
+    return !(length($_[0]) == bytes::length($_[0]))
+} unless defined &utf8::is_utf8;
+
 $DBI::PurePerl = $ENV{DBI_PUREPERL} || 1;
 $DBI::PurePerl::VERSION = sprintf "%d.%02d", '$Revision: 1.96 $ ' =~ /(\d+)\.(\d+)/;
 $DBI::neat_maxlen ||= 400;
 
 $DBI::tfh = Symbol::gensym();
 open $DBI::tfh, ">&STDERR" or warn "Can't dup STDERR: $!";
+select( (select($DBI::tfh), $| = 1)[0] );  # autoflush
 
 
 %DBI::last_method_except = map { $_=>1 } qw(DESTROY _set_fbav set_err);
@@ -301,6 +309,10 @@ sub  _install_method {
     push @post_call_frag, q{
 	$keep_error = 0 if $keep_error && $h->{ErrCount} > $ErrCount;
 
+	$DBI::err    = $h->{err};
+	$DBI::errstr = $h->{errstr};
+	$DBI::state  = $h->{state};
+
         if ( !$keep_error
 	&& defined(my $err = $h->{err})
 	&& ($call_depth <= 1 && !$h->{_parent}{_call_depth})
@@ -329,7 +341,7 @@ sub  _install_method {
 		    }
 		    $msg .= "]";
 		}
-		if ($DBI::err eq "0") { # is 'warning' (not info)
+		if ($err eq "0") { # is 'warning' (not info)
 		    carp $msg if $pw;
 		}
 		else {
@@ -394,7 +406,7 @@ sub  _install_method {
     warn "$@\n$method_code\n" if $@;
     die "$@\n$method_code\n" if $@;
     *$method = $code_ref;
-    if (0 && $method =~ /set_err/) { # debuging tool
+    if (0 && $method =~ /do/) { # debuging tool
 	my $l=0; # show line-numbered code for method
 	warn "*$method = ".join("\n", map { ++$l.": $_" } split/\n/,$method_code);
     }
@@ -521,7 +533,7 @@ sub hash {
     elsif ($type == 1) {	# Fowler/Noll/Vo hash
         # see http://www.isthe.com/chongo/tech/comp/fnv/
         require Math::BigInt;   # feel free to reimplement w/o BigInt!
-	my $version = $Math::BigInt::VERSION || 0;
+	(my $version = $Math::BigInt::VERSION || 0) =~ s/_.*//; # eg "1.70_01"
 	if ($version >= 1.56) {
 	    $hash = Math::BigInt->new(0x811c9dc5);
 	    for my $uchar (unpack ("C*", $key)) {
@@ -551,16 +563,18 @@ sub looks_like_number {
     }
     return (@_ >1) ? @new : $new[0];
 }
+
 sub neat {
     my $v = shift;
     return "undef" unless defined $v;
     return $v if (($v & ~ $v) eq "0"); # is SvNIOK
+    my $quote = utf8::is_utf8($v) ? '"' : "'";
     my $maxlen = shift || $DBI::neat_maxlen;
     if ($maxlen && $maxlen < length($v) + 2) {
 	$v = substr($v,0,$maxlen-5);
 	$v .= '...';
     }
-    return "'$v'";
+    return "$quote$v$quote";
 }
 
 package
@@ -576,6 +590,13 @@ sub FETCH {
 package
 	DBD::_::common;
 
+sub swap_inner_handle {
+    my ($h1, $h2) = @_;
+    # can't make this work till we can get the outer handle from the inner one
+    # probably via a WeakRef
+    return $h1->set_err(1, "swap_inner_handle not currently supported by DBI::PurePerl");
+}
+
 sub trace {	# XXX should set per-handle level, not global
     my ($h, $level, $file) = @_;
     $level = $h->parse_trace_flags($level)
@@ -827,7 +848,7 @@ __END__
 
 =head1 NAME
 
- DBI::PurePerl -- a DBI emulation using pure perl (no C/XS compilation required)
+DBI::PurePerl -- a DBI emulation using pure perl (no C/XS compilation required)
 
 =head1 SYNOPSIS
 
@@ -926,7 +947,7 @@ is defined.  To enable total tracing you can set the DBI_TRACE
 environment variable as usual.  But to enable individual handle
 tracing using the trace() method you also need to set the DBI_TRACE
 environment variable, but set it to 0.
- 
+
 =head2 Parameter Usage Checking
 
 The DBI does some basic parameter count checking on method calls.
